       subroutine apoisson(l,n,h,q,den,rv)
!  use Numerov algorithm to solve poisson equation
!  for angularly dependent charge distribution of angular momentum l
!  den(n) is electron density * (4*pi*r**2) appropriate for l
!  rv(n) is returned as electrostatic potential * r
!  q is returned as the lth moment of the charge
!  a(n), b(n), and c(n) are work arrays
       implicit none
       integer, intent(IN) :: l,n
       real*8, intent(IN) :: h,den(n)
       real*8, intent(INOUT) :: q
       real*8, intent(OUT) :: rv(n)

       integer :: i
       real*8 :: angm,r,overint
       real*8, allocatable :: a(:),b(:),c(:)

       allocate(a(n),b(n),c(n),stat=i)
        if (i/=0) then
         write(6,*) 'Error in allocating arrays in apoisson ',i,n
         stop
        endif

       do i=1,n
        a(i)=2*h*den(i)/(i)
        b(i)=den(i)*((i*h)**l)
       enddo
       q=overint(n,h,b)/(2*l+1)
       rv(1)=a(1)+0.1d0*a(2)
       do i=2,n-1
        rv(i)=a(i)+0.1d0*(a(i+1)+a(i-1))
       enddo
       rv(n)=a(n)+0.1d0*a(n-1)
       angm=l*(l+1)
       r=(n+1)*h
       rv(n)=rv(n)+(2.4d0-0.2d0*angm/((n+1)**2))*q/(r**l)
       do i=1,n
        a(i)=angm/(i*i)
        b(i)=2.4d0+a(i)
       enddo
       do i=1,n-1
        c(i)=-1.2d0+0.1d0*a(i+1)
       enddo
       do i=n,2,-1
        a(i)=-1.2d0+0.1d0*a(i-1)
       enddo
! use Thomas's algorithm for inverting matrix
       call thomas(n,a,b,c,rv)
!
       deallocate(a,b,c)
      return
       end
