       subroutine poisson(n,h,q,den,rv,ecoul)
!  use Numerov algorithm to solve poisson equation
!  den(n) is electron density * (4*pi*r**2)
!  rv(n) is returned as electrostatic potential * r
!  q is total electron charge
!  a(n) and b(n) are work arrays
!  ecoul is the coulomb interation energy
       IMPLICIT NONE
       integer, intent(IN)::n
       real*8, intent(IN):: h,q,den(n)
       real*8, intent(OUT) :: rv(n),ecoul

       real*8, allocatable :: a(:),b(:)
       real*8 :: sd,sl,ss2,overint,th
       integer :: i

      allocate(a(n),b(n),stat=i)
       if (i/=0) then
        write(6,*) 'Error in allocating arrays in poisson',i,n
        stop
       endif

       sd=2
       sl=-1
       do i=1,n
       a(i)=h*den(i)/(6*i)
       enddo
       rv(1)=10*a(1)+a(2)
       do i=2,n-1
       rv(i)=10*a(i)+a(i+1)+a(i-1)
       enddo
       rv(n)=10*a(n)+a(n-1)+2*q
! use Thomas's algorithm for inverting matrix
!   Dale U. von Rosenberg, "Methods for the Numerical Solution of
!     Partial Differential Equations, Am. Elsevier Pub., 1969, pg. 113
      a(1)=sd
      ss2=sl*sl
      do i=2,n
      a(i)=sd-ss2/a(i-1)
      enddo
      b(1)=rv(1)/sd
      do i=2,n
      b(i)=(rv(i)-sl*b(i-1))/a(i)
      enddo
      rv(n)=b(n)
      do i=n-1,1,-1
      rv(i)=b(i)-sl*rv(i+1)/a(i)
      enddo
!
!
!  calculate ecoul
!
       do i=1,n
       a(i)=den(i)*rv(i)/i
       enddo
       th=1
       ecoul=0.5d0*overint(n,th,a)
       write(6,*) 'ecoul = ',ecoul
       deallocate(a,b)
!
      return
       end
